# QRSSaudioSSB-v01.py (24-07-2012)
import math
import time
import array
import wave
import pyaudio
import sys
import struct

from time import gmtime, strftime
from Tkinter import *
from tkFileDialog import askopenfilename
from tkFileDialog import asksaveasfilename
from tkSimpleDialog import askstring
from tkMessageBox import *

############################################################################################################################################
# Settings that can be modified
AUDIOdevin = None           # Audio device for input. None = Windows default. Can be modified with the Audio device button.
AUDIOdevout = None          # Audio device for output. None = Windows default. Can be modified with the Audio device button.
CENTERfrequency = 1495      # Center frequency of the reception band
NEWCENTERfrequency = 45     # The new center frequency of the down mixed reception band
BANDWIDTH = 10              # The reception bandwith
FILTER = 1                  # 1=Very simple moving average FIR filter with variable bandwidth
                            # 2=IIR filter, 4 pole Butterworth, 100 Hz bandwidth ONLY for sample rate=6000!
                            # 3=IIR filter, 4 pole Butterworth, 150 Hz bandwidth ONLY for sample rate=6000!
                            # 4=IIR filter, 4 pole Butterworth, 50 Hz bandwidth ONLY for sample rate=6000!
MULTIFILTER = True          # If True: 3x FIR filter or 2x IIR filter after each other instead of one
NEWspeed = 16               # Multiply the speed of the old wav file with this factor
NEWvolume = 20              # New volume of converted WAV file

SAMPLERATE = 6000           # DEFAULT=6000, has to be 6000 for the IIR filters!!! Can be changed if only the FIR filter is used

############################################################################################################################################
# Initialisation of global variables required in various routines (DO NOT MODIFY THEM!)

STOP = False                # Stop key pressed
############################################################################################################################################

# ============== Start widgets routines =====================
def Bnot():
    print "Routine not made yet"


def Bstop():
    global STOP
    STOP = True


def BRECORDONEwav():    # Record only one file, this routine is not used in the program.
    global STOP

    filename = ASKWAVsavefilename()

    if filename == "":
        s = "\n\nRecord WAV file cancelled"   
        text1.insert(END, s)    
        return()

    s = askstring("Record seconds","Value (1 - 10000):")

    if (s == None):     # No input, cancel or error
        s = "0"
      
    try:                #Error if no numeric characters
        count = int(s)
    except:
        count = 0
        
    if (count < 1) or (count > 10000):
        s = "\n\nRecording time out of range, cancelled"   
        text1.insert(END, s)    
        return()

    STOP = False
    RECORDwav(filename, count)


def BCONVERTwav():
    global STOP
    
    infile = ASKWAVinfilename()
    if infile == "":
        return
    
    outfile = ASKWAVsavefilename()
    if outfile == "":
        return

    if infile == outfile:
        s = "\n\nConversion cancelled, input WAV file name should not be the same as the output WAV file name!"  
        text1.insert(END, s)    
        root.update()
        return
        
    ASKconversionvariables()

    Start = askyesno("Start Conversion", "Start Conversion?",default = YES)

    if Start == True:
        STOP = False
        CONVERTwav(infile, outfile)


def BRECORDwav():
    RECORDCONVERTwav(False)     # Recording only, no conversion


def BRECORDCONVERTwav():
    RECORDCONVERTwav(True)      # Recording and conversion
    

def BPLAYwav():
    global STOP
        
    filename = ASKWAVinfilename()

    if (filename == ""):
        s = "\n\nPlay WAV file cancelled"   
        text1.insert(END, s)    
        return()

    STOP = False
    PLAYwav(filename)


# =================================================================================

def RECORDCONVERTwav(CONVERT):
    global STOP

    RECORDTIME = 1200                               # Record time is 20 minutes (1200 sec.)
            
    if CONVERT == True:                             # If False then only recording, no conversion
        ASKconversionvariables()
        s = "\n\nRecording and Converting"
    else:
        s = "\n\nRecording"

    text1.insert(END, s)    

    STOP = False
    while STOP == False:
        root.update_idletasks()
        root.update()

        tme =  strftime("%Y%b%d-%H%M", gmtime())    # The time
        infile = "WAV" + tme + ".wav"
        outfile = "CONV-" + infile

        RECORDwav(infile, RECORDTIME)

        if CONVERT == True and STOP == False:       # If CONVERT=False then only recording, no conversion
            CONVERTwav(infile, outfile)


def RECORDwav(filename, count):
    global STOP
    global AUDIOdevin
    global AUDIOdevout
    global SAMPLERATE

    FORMAT = pyaudio.paInt16
    CHANNELS = 1
    RATE = int(SAMPLERATE)
    RECORD_SECONDS = int(count)
    WAVrecordfilename = filename
    
    PA = pyaudio.PyAudio()

    stream = PA.open(format = FORMAT,
                channels = CHANNELS,
                rate = RATE,
                input = True,
                output = True,
                frames_per_buffer = RATE,
                input_device_index = AUDIOdevin,
                output_device_index = AUDIOdevout)

    s = "\n\nRecording: " + WAVrecordfilename   
    text1.insert(END, s)    

    root.update() # update screens
    
    all = []
    n = 0
    while n < RECORD_SECONDS and STOP == False:
        data = stream.read(RATE)
        stream.write(data, RATE)
        all.append(data)
        root.update_idletasks()
        root.update()
        n = n + 1

    stream.close()
    PA.terminate()

    # write data to WAVE file
    data = ''.join(all)
    wf = wave.open(WAVrecordfilename, 'wb')
    wf.setnchannels(CHANNELS)
    wf.setsampwidth(PA.get_sample_size(FORMAT))
    wf.setframerate(RATE)
    wf.writeframes(data)
    wf.close()

    s = "\nDone!"
    text1.insert(END, s)    


def PLAYwav(filename):
    global STOP
    global AUDIOdevout

    WAVplayfilename = filename
    chunk = 12000                               # Buffer size
   
    wf = wave.open(WAVplayfilename, 'rb')

    PA = pyaudio.PyAudio()

    # open stream
    stream = PA.open(format =
                PA.get_format_from_width(wf.getsampwidth()),
                channels = wf.getnchannels(),
                rate = wf.getframerate(),
                output = True,
                output_device_index = AUDIOdevout)

    # read data
    data = wf.readframes(chunk)

    s = "\n\nPlaying: " + WAVplayfilename   
    text1.insert(END, s)    

    # play stream
    while data != '' and STOP == False:
        stream.write(data)
        data = wf.readframes(chunk)
        root.update_idletasks()
        root.update()

    stream.close()
    PA.terminate()

    s = "\nDone!"
    text1.insert(END, s)    


def CLRscreens():
    text1.delete(1.0, END)
    s = "Log screen"  
    text1.insert(END, s)    


def CONVERTwav(infile, outfile):
    global CENTERfrequency
    global NEWCENTERfrequency
    global BANDWIDTH
    global FILTER
    global MULTIFILTER
    global NEWspeed
    global NEWvolume
    global STOP

    T1 = time.time()
    
    WAVsignal1 = []
    WAVsignal2 = []

    WAVinfilename = infile
    if WAVinfilename == "":
        return
    
    WAVoutfilename = outfile
    if WAVoutfilename == "":
        return
        
    s = "\n\nReading: " + WAVinfilename   
    text1.insert(END, s)    
    root.update_idletasks()
    root.update()

    WAVf = wave.open(WAVinfilename, 'rb')
    WAVframes = WAVf.getnframes()
    # print "frames: ", WAVframes
    WAVchannels = WAVf.getnchannels()
    # print "channels: ", WAVchannels
    WAVsamplewidth = WAVf.getsampwidth()
    # print "samplewidth: ", WAVsamplewidth
    WAVframerate = WAVf.getframerate()
    # print "framerate: ", WAVframerate
    SAMPLErate = WAVframerate

    signals = WAVf.readframes(WAVframes)        # Read the data from the WAV file and convert to WAVsignalx[]
                
    i = 0
    f = 0
    s = ""

    WAVsignal1 = []
    WAVsignal2 = []

    if (WAVsamplewidth == 1) and (WAVchannels == 1):
        while (f < WAVframes):
            s = str(struct.unpack('B', signals[i:(i+1)]))
            v = int(s[1:-2]) - 128
            WAVsignal1.append(v) 
            WAVsignal2.append(0) 
            i = i + 1
            f = f + 1
                    
    if (WAVsamplewidth == 1) and (WAVchannels == 2):
        while (f < WAVframes):
            s = str(struct.unpack('B', signals[i:(i+1)]))
            v = int(s[1:-2]) - 128
            WAVsignal1.append(v) 
            s = str(struct.unpack('B', signals[(i+1):(i+2)]))
            v = int(s[1:-2])
            WAVsignal2.append(v) 
            i = i + 2
            f = f + 1

    if (WAVsamplewidth == 2) and (WAVchannels == 1):
        while (f < WAVframes):
            s = str(struct.unpack('h', signals[i:(i+2)]))
            v = int(s[1:-2])
            WAVsignal1.append(v) 
            WAVsignal2.append(0) 
            i = i + 2
            f = f + 1

    if (WAVsamplewidth == 2) and (WAVchannels == 2):
        while (f < WAVframes):
            s = str(struct.unpack('h', signals[i:(i+2)]))
            v = int(s[1:-2])
            WAVsignal1.append(v) 
            s = str(struct.unpack('h', signals[(i+2):(i+4)]))
            v = int(s[1:-2])
            WAVsignal2.append(v) 
            i = i + 4
            f = f + 1

    WAVf.close()

    if STOP == True:
        return
    
    # Downmixing
    s = "\nDownmixing: " + WAVinfilename   
    text1.insert(END, s)    
    root.update_idletasks()
    root.update()

    # The SSB receiver in accordance with the Weaver method, suppression of lower side band
    if FILTER == 1:                                     # Moving Average FIR filter
        if MULTIFILTER == True:
            B = 0.8                                     # The bandwidth of the 3 Moving Average FIR filters after each other
        else:
            B = 1.0                                     # The bandwidth of one Moving Average FIR filter

        # Taps = B x Samplerate / (2 x F3db) For the Weaver principle, half the bandwidth is required!
        s1 = B * float(SAMPLErate) / BANDWIDTH          # s1 is number of Taps required
        s1 = int(s1)
        s2 = float(s1) / 1.5                            # If the 2nd has 1.5x FIR1 bandwidth, the first zero suppresses the 1nd maximum of FIR1
        s2 = int(s2)
        s3 = float(s2) / 1.5                            # If the 3nd has 1.5x FIR2 bandwidth, the first zero suppresses the 1nd maximum of FIR2
        s3 = int(s3)
                
        if MULTIFILTER == True:
            print "Filtering and down mixing, Moving Average FIR MULTIFILTER, taps: ", s1
        else:
            print "Filtering and down mixing, Moving Average FIR filter, taps: ", s1

        I1 = []
        Q1 = []
        n = 0
        while n < s1:                                   # Fill the FIR1 circular sample memory the I and Q channel with zeros
            I1.append(0.0)
            Q1.append(0.0)
            n = n + 1

        I2 = []
        Q2 = []
        n = 0
        while n < s2:                                   # Fill the FIR2 circular sample memory the I and Q channel with zeros
            I2.append(0.0)
            Q2.append(0.0)
            n = n + 1

        I3 = []
        Q3 = []
        n = 0
        while n < s3:                                   # Fill the FIR3 circular sample memory the I and Q channel with zeros
            I3.append(0.0)
            Q3.append(0.0)
            n = n + 1

        TWOPI = 2.0 * math.pi
        ST1 = TWOPI * float(CENTERfrequency) / SAMPLErate           # The center frequency of the audio band
        H1 = 0.0
        ST2 = TWOPI * float(NEWCENTERfrequency) / SAMPLErate        # Center frequency2 = NEWCENTERfrequency
        H2 = 0.0

        VI1 = int(0)                                    # The sum of the samples to be averaged, I channel FIR1
        VQ1 = int(0)                                    # The sum of the samples to be averaged, Q channel FIR1
        VI2 = int(0)                                    # The sum of the samples to be averaged, I channel FIR2
        VQ2 = int(0)                                    # The sum of the samples to be averaged, Q channel FIR2
        VI3 = int(0)                                    # The sum of the samples to be averaged, I channel FIR3
        VQ3 = int(0)                                    # The sum of the samples to be averaged, Q channel FIR3
        pntr1 = 0                                       # The pointer in the FIR1 circular sample memory
        pntr2 = 0                                       # The pointer in the FIR2 circular sample memory
        pntr3 = 0                                       # The pointer in the FIR3 circular sample memory 
        L = len(WAVsignal1)                             # L is the length of the audio data array
        n = 0
        while n < L:
            vi = WAVsignal1[n] * math.sin(H1)           # The first I mixer of the Weaver SSB receiver
            vq = WAVsignal1[n] * math.cos(H1)           # The first Q mixer of the Weaver SSB receiver      
            H1 = H1 + ST1
            while H1 > TWOPI:
                H1 = H1 - TWOPI

            I1[pntr1] = int(vi)                         # Fill the circular I memory of FIR1
            Q1[pntr1] = int(vq)                         # Fill the circular Q memory of FIR1      
                        
            VI1 = VI1 + I1[pntr1]                       # Add the new sample to the I Moving Average FIR1 filter memory
            VQ1 = VQ1 + Q1[pntr1]                       # Add the new sample to the Q Moving Average FIR1 filter memory

            pntr1 = pntr1 + 1                           # Circular memory pointer increment
            if pntr1 == s1:
                pntr1 = 0
                      
            VI1 = VI1 - I1[pntr1]                       # Substract the oldest sample from the I Moving Average FIR1 filter memory
            VQ1 = VQ1 - Q1[pntr1]                       # Substract the oldest sample from the Q Moving Average FIR1 filter memory

            vi = float(VI1) / s1                        # The average of s samples I path of the Weaver SSB receiver
            vq = float(VQ1) / s1                        # The average of s samples Q path of the Weaver SSB receiver
                    
            if MULTIFILTER == True:                     # If True, also a 2nd and 3rd FIR filter added
                I2[pntr2] = int(vi)                     # Fill the circular I memory of FIR2
                Q2[pntr2] = int(vq)                     # Fill the circular Q memory of FIR2
                        
                VI2 = VI2 + I2[pntr2]                   # Add the new sample to the I Moving Average FIR2 filter memory
                VQ2 = VQ2 + Q2[pntr2]                   # Add the new sample to the Q Moving Average FIR2 filter memory

                pntr2 = pntr2 + 1                       # Circular memory pointer increment
                if pntr2 == s2:
                    pntr2 = 0
                          
                VI2 = VI2 - I2[pntr2]                   # Substract the oldest sample from the I Moving Average FIR2 filter memory
                VQ2 = VQ2 - Q2[pntr2]                   # Substract the oldest sample from the Q Moving Average FIR2 filter memory

                vi = float(VI2) / s2                    # The average of s samples I path of the Weaver SSB receiver
                vq = float(VQ2) / s2                    # The average of s samples Q path of the Weaver SSB receiver

                I3[pntr3] = int(vi)                     # Fill the circular I memory of FIR3
                Q3[pntr3] = int(vq)                     # Fill the circular Q memory of FIR3
                        
                VI3 = VI3 + I3[pntr3]                   # Add the new sample to the I Moving Average FIR3 filter memory
                VQ3 = VQ3 + Q3[pntr3]                   # Add the new sample to the Q Moving Average FIR3 filter memory

                pntr3 = pntr3 + 1                       # Circular memory pointer increment
                if pntr3 == s3:
                    pntr3 = 0
                          
                VI3 = VI3 - I3[pntr3]                   # Substract the oldest sample from the I Moving Average FIR3 filter memory
                VQ3 = VQ3 - Q3[pntr3]                   # Substract the oldest sample from the Q Moving Average FIR3 filter memory

                vi = float(VI3) / s3                    # The average of s samples I path of the Weaver SSB receiver
                vq = float(VQ3) / s3                    # The average of s samples Q path of the Weaver SSB receiver

            v = vi * math.sin(H2) + vq * math.cos(H2)   # The second mixer and adder in the Weaver SSB receiver
            H2 = H2 + ST2

            while H2 > TWOPI:  
                H2 = H2 - TWOPI

            WAVsignal1[n] = int (v)

            n = n + 1

    if FILTER > 1: 
        # 4 pole Butterworth low pass IIR filter http://www-users.cs.york.ac.uk/~fisher/mkfilter/trad.html
        # ONLY for sample rate = 6000!!!
        # 50 Hz bandwidth required for 100 Hz reception bandwidth in the Weaver SSB receiver.
        # 25 Hz bandwidth required for 50 Hz reception bandwidth in the Weaver SSB receiver.

        if MULTIFILTER == True:
            print "Filtering and down mixing, IIR MULTIFILTER"
        else:
            print "Filtering and down mixing, IIR filter"

        TWOPI = 2.0 * math.pi
        ST1 = TWOPI * float(CENTERfrequency) / SAMPLErate           # The center frequency of the audio band
        H1 = 0.0
        ST2 = TWOPI * float(NEWCENTERfrequency) / SAMPLErate        # Center frequency2 = NEWCENTERfrequency
        H2 = 0.0

        x1vi0 = 0    # The taps and memory for the I path IIR1 filter
        x1vi1 = 0
        x1vi2 = 0
        x1vi3 = 0
        x1vi4 = 0
        y1vi0 = 0
        y1vi1 = 0
        y1vi2 = 0
        y1vi3 = 0
        y1vi4 = 0

        x1vq0 = 0    # The taps and memory for the Q path IIR1 filter
        x1vq1 = 0
        x1vq2 = 0
        x1vq3 = 0
        x1vq4 = 0
        y1vq0 = 0
        y1vq1 = 0
        y1vq2 = 0
        y1vq3 = 0
        y1vq4 = 0

        x2vi0 = 0    # The taps and memory for the I path IIR2 filter
        x2vi1 = 0
        x2vi2 = 0
        x2vi3 = 0
        x2vi4 = 0
        y2vi0 = 0
        y2vi1 = 0
        y2vi2 = 0
        y2vi3 = 0
        y2vi4 = 0

        x2vq0 = 0    # The taps and memory for the Q path IIR2 filter
        x2vq1 = 0
        x2vq2 = 0
        x2vq3 = 0
        x2vq4 = 0
        y2vq0 = 0
        y2vq1 = 0
        y2vq2 = 0
        y2vq3 = 0
        y2vq4 = 0

        if FILTER == 2:             # Gain and coefficients for the 50 Hz 4 pole Butterworth IIR filter and 6000 samples/sec.
            GAIN = 2.277420645e+06
            C1 = -0.8721086451
            C2 = 3.6077524539
            C3 = -5.5988354567
            C4 = 3.8631846224

        if FILTER == 3:             # Gain and coefficients for the 75 Hz 4 pole Butterworth IIR filter and 6000 samples/sec.
            GAIN = 4.649932749e+05
            C1 = -0.8144059977
            C2 = 3.4247473473
            C3 = -5.4051668617
            C4 = 3.7947911031

        if FILTER == 4:             # Gain and coefficients for the 25 Hz 4 pole Butterworth IIR filter and 6000 samples/sec.
            GAIN = 3.523725849e+07
            C1 = -0.9338739884
            C2 = 3.7993827672
            C3 = -5.7970987038
            C4 = 3.9315894710
            
        n = 0
        L = len(WAVsignal1)
        while n < L:
            vi = float(WAVsignal1[n]) * math.sin(H1)      # The first I mixer of the Weaver SSB receiver
            vq = float(WAVsignal1[n]) * math.cos(H1)      # The first Q mixer of the Weaver SSB receiver      
            H1 = H1 + ST1
            while H1 > TWOPI:
                H1 = H1 - TWOPI

            # The IIR1 routine and coefficients from http://www-users.cs.york.ac.uk/~fisher/mkfilter/trad.html
            x1vi0 = x1vi1
            x1vi1 = x1vi2
            x1vi2 = x1vi3
            x1vi3 = x1vi4
            x1vi4 = vi / GAIN
            y1vi0 = y1vi1
            y1vi1 = y1vi2
            y1vi2 = y1vi3
            y1vi3 = y1vi4
            y1vi4 = (x1vi0 + x1vi4) + 4 * (x1vi1 + x1vi3) + 6 * x1vi2 + (C1 * y1vi0) + (C2 * y1vi1) + (C3 * y1vi2) + (C4 * y1vi3)
            vi = y1vi4

            x1vq0 = x1vq1
            x1vq1 = x1vq2
            x1vq2 = x1vq3
            x1vq3 = x1vq4
            x1vq4 = vq / GAIN
            y1vq0 = y1vq1
            y1vq1 = y1vq2
            y1vq2 = y1vq3
            y1vq3 = y1vq4
            y1vq4 = (x1vq0 + x1vq4) + 4 * (x1vq1 + x1vq3) + 6 * x1vq2 + (C1 * y1vq0) + (C2 * y1vq1) + (C3 * y1vq2) + (C4 * y1vq3)
            vq = y1vq4

            # The IIR2 routine and coefficients from http://www-users.cs.york.ac.uk/~fisher/mkfilter/trad.html
            if MULTIFILTER == True:                     # If True, also a 2nd IIR filter added
                x2vi0 = x2vi1
                x2vi1 = x2vi2
                x2vi2 = x2vi3
                x2vi3 = x2vi4
                x2vi4 = vi / GAIN
                y2vi0 = y2vi1
                y2vi1 = y2vi2
                y2vi2 = y2vi3
                y2vi3 = y2vi4
                y2vi4 = (x2vi0 + x2vi4) + 4 * (x2vi1 + x2vi3) + 6 * x2vi2 + (C1 * y2vi0) + (C2 * y2vi1) + (C3 * y2vi2) + (C4 * y2vi3)
                vi = y2vi4

                x2vq0 = x2vq1
                x2vq1 = x2vq2
                x2vq2 = x2vq3
                x2vq3 = x2vq4
                x2vq4 = vq / GAIN
                y2vq0 = y2vq1
                y2vq1 = y2vq2
                y2vq2 = y2vq3
                y2vq3 = y2vq4
                y2vq4 = (x2vq0 + x2vq4) + 4 * (x2vq1 + x2vq3) + 6 * x2vq2 + (C1 * y2vq0) + (C2 * y2vq1) + (C3 * y2vq2) + (C4 * y2vq3)
                vq = y2vq4

            v = vi * math.sin(H2) + vq * math.cos(H2)   # The second mixer and adder in the Weaver SSB receiver
            H2 = H2 + ST2

            while H2 > TWOPI:
                H2 = H2 - TWOPI

            WAVsignal1[n] = int (v)
                    
            n = n + 1

    root.update_idletasks()
    root.update()
    if STOP == True:
        return
        
    # Speedconversion and save
    s = "\nSpeedconversion: " + WAVinfilename   
    text1.insert(END, s)    
    root.update_idletasks()
    root.update()

    s = []
    L = len(WAVsignal1)
   
    n = 0
    while n < L:
        v = float(WAVsignal1[n]) * NEWvolume            # Get the sample and multiply with the new volume

        if v < -32000:                                  # Clipping!
            v = -32000.0
        if v > 32000:
            v = 32000.0
            
        if v < 0:                                       # Ones complement
            v = v + 65536
        vi = int(v)    
        s.append(int(vi % 256))
        s.append(int(vi / 256))

        n = n + NEWspeed
        
    ssignal = ''                                        # Buffer for list of binary values
    ssignal = array.array('B', s).tostring()            # Convert int to binary string with array module
    OUTwavfile(WAVoutfilename, SAMPLErate, ssignal)     # Write binary list to WAV file

    s = "\nConversion of: " + WAVinfilename + " to: " + WAVoutfilename + " Done!"  
    text1.insert(END, s)    
    root.update_idletasks()
    root.update()

    T2 = time.time()
    print "Conversion time: ", int(T2-T1), " s"


def OUTwavfile(WAVname, samplerate, ssignal):           # Save WAV file
    WAVf = wave.open(WAVname, 'wb')
    WAVf.setparams((1, 2, samplerate, 1, 'NONE', 'noncompressed'))
    WAVf.writeframes(ssignal)
    WAVf.close()


def ASKWAVinfilename():
    filename = askopenfilename(filetypes=[("WAVfile","*.wav"),("allfiles","*")])

    if (filename == None):              # No input, cancel pressed or an error
        filename = ""

    if (filename == ""):
        return(filename)
    
    if filename[-4:] != ".wav":
        filename = filename + ".wav"

    return(filename)


def ASKWAVsavefilename():
    filename = asksaveasfilename(filetypes=[("wav files","*.wav"),("allfiles","*")])

    if (filename == None):              # No input, cancel pressed or an error
        filename = ""

    if (filename == ""):
        return(filename)
    
    if filename[-4:] != ".wav":
        filename = filename + ".wav"

    return(filename)


def ASKconversionvariables():
    global CENTERfrequency
    global NEWCENTERfrequency
    global BANDWIDTH
    global FILTER
    global MULTIFILTER
    global NEWspeed
    global NEWvolume

    # Filter type
    s = askstring("Filter type: ","FILTER: " + str(FILTER) + "\n\n1=FIR moving average\n2=IIR 100 Hz bandwidth\n3=IIR 150 Hz bandwidth\n4=IIR 50 Hz bandwidth")

    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error":
        v = int(s)

        if v > 0 and v < 5:
            FILTER = v

    if FILTER > 1 and SAMPLERATE != 6000:
        showwarning("WARNING","Sample rate has to be 6000 for FIR filters!")

    if FILTER == 1:         # Bandwidth only for filter 1!
        s = askstring("Reception bandwidth: ","Value: " + str(BANDWIDTH) + " Hz\n\nNew value:\n")
        
        try:                # Error if for example no numeric characters or OK pressed without input (s = "")
            v = int(s)
        except:
            s = "error"

        if s != "error":
            BANDWIDTH = abs(v)


    # Multifilter if True: 3x FIR filter or 2x IIR filter after each other instead of one
    MULTIFILTER = askyesno("Enable Multifilter", "Enable Multifilter?",default = NO)


    # Center frequency
    s = askstring("Center frequency: ","Value: " + str(CENTERfrequency) + " Hz\n\nNew value:\n")
    
    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error":
        CENTERfrequency = abs(v)

    # New center frequency (center frequency of audio filter)
    s = askstring("New center frequency: ","Value: " + str(NEWCENTERfrequency) + " Hz\n\nNew value:\n")
    
    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error":
        NEWCENTERfrequency = abs(v)

    # New high play speed
    s = askstring("New speed (16): ","Value: " + str(NEWspeed) + "x\n\nNew value:\n")
    
    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error" and v >= 1:
        NEWspeed = v

    # New volume
    s = askstring("New volume (1-100): ","Value: " + str(NEWvolume) + "x\n\nNew value:\n")
    
    try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
        v = int(s)
    except:
        s = "error"

    if s != "error" and v >= 1:
        NEWvolume = v


def SELECTaudiodevice():        # Select an audio device
    global AUDIOdevin
    global AUDIOdevout

    PA = pyaudio.PyAudio()
    ndev = PA.get_device_count()

    n = 0
    ai = ""
    ao = ""
    while n < ndev:
        s = PA.get_device_info_by_index(n)
        # print n, s
        if s['maxInputChannels'] > 0:
            ai = ai + str(s['index']) + ": " + s['name'] + "\n"
        if s['maxOutputChannels'] > 0:
            ao = ao + str(s['index']) + ": " + s['name'] + "\n"
        n = n + 1
    PA.terminate()

    AUDIOdevin = None
    
    s = askstring("Device","Select audio INPUT device:\nPress Cancel for Windows Default\n\n" + ai + "\n\nNumber: ")
    if (s != None):             # If Cancel pressed, then None
        try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
            v = int(s)
        except:
            s = "error"

        if s != "error":
            if v < 0 or v > ndev:
                v = 0
            AUDIOdevin = v

    AUDIOdevout = None

    s = askstring("Device","Select audio OUTPUT device:\nPress Cancel for Windows Default\n\n" + ao + "\n\nNumber: ")
    if (s != None):             # If Cancel pressed, then None
        try:                    # Error if for example no numeric characters or OK pressed without input (s = "")
            v = int(s)
        except:
            s = "error"

        if s != "error":
            if v < 0 or v > ndev:
                v = 0
            AUDIOdevout = v


# ================ Start Make Screen ==========================

root=Tk()
root.title("QRSSaudioSSB-v01.py (24-07-2012): Listening to QRSS signals")

root.minsize(10, 10)

frame1 = Frame(root, background="blue", borderwidth=5, relief=RIDGE)
frame1.pack(side=TOP, expand=1, fill=X)

frame1a = Frame(root, background="blue", borderwidth=5, relief=RIDGE)
frame1a.pack(side=TOP, expand=1, fill=X)

frame3 = Frame(root, background="red", borderwidth=5, relief=RIDGE)
frame3.pack(side=TOP, expand=1, fill=X)

scrollbar1 = Scrollbar(frame1)
scrollbar1.pack(side=RIGHT, expand=YES, fill=BOTH)

text1 = Text(frame1, height=20, width=130, yscrollcommand=scrollbar1.set)
text1.pack(side=TOP, expand=1, fill=X)

scrollbar1.config(command=text1.yview)

b = Button(frame1a, text="RECORDwav", width=30, command=BRECORDwav)
b.pack(side=LEFT, padx=5, pady=5)

b = Button(frame1a, text="CONVERTwav", width=30, command=BCONVERTwav)
b.pack(side=LEFT, padx=5, pady=5)

b = Button(frame1a, text="RECORD & CONVERTwav", width=30, command=BRECORDCONVERTwav)
b.pack(side=LEFT, padx=5, pady=5)

b = Button(frame1a, text="STOP", width=30, background="red", command=Bstop)
b.pack(side=RIGHT, padx=5, pady=5)

b = Button(frame1a, text="PLAYwav", width=30, command=BPLAYwav)
b.pack(side=RIGHT, padx=5, pady=5)

# label1 = Label(frame1, text=inhoud, justify=LEFT, width=60, height=1)
# label1.pack(side=TOP, anchor=W)

b = Button(frame3, text="CLEAR SCREEN", width=15, command=CLRscreens)
b.pack(side=TOP, expand=1, fill=X, padx=2, pady=2)


# ================ Main routine ================================================
s = "Log screen"  
text1.insert(END, s)    
root.update_idletasks()
root.update()

SELECTaudiodevice()         # Select an audio device

mainloop()



